<?php

namespace EWR\Discord\Repository;

use XF\Mvc\Entity\AbstractCollection;
use XF\Mvc\Entity\Repository;

class Discord extends Repository
{
	public function assertSetup()
	{
		$provider = $this->getProvider();
			
		if (empty($provider->options['client_id']) ||
			empty($provider->options['client_secret']) ||
			empty($provider->options['bot_token']))
		{
			return false;
		}
		
		return $provider;
	}
	
	public function getGuild($endpoint = '')
	{
		$provider = $this->assertSetup();
		$server = $this->getServer();
		
		if (!$provider || !$server)
		{
			return false;
		}
		
		try
		{
			$client = \XF::app()->http()->client(['timeout' => 5]);
			$response = $client->get('https://discord.com/api/guilds/'.$server.$endpoint, [
				'headers' => [
					'Authorization' => 'Bot '.$provider->options['bot_token'],
					'User-Agent' => 'DiscordBot (8wayrunBot, v2)',
				],
				'query' => [
					'limit' => 1000,
				],
			]);
			$json = \GuzzleHttp\json_decode($response->getBody(), true);
		}
		catch (\Exception $e)
		{
			if ($e->hasResponse())
			{
				$status = $e->getResponse()->getStatusCode();
				
				if ($status == 404 || $status == 500)
				{
					return false;
				}
			}
			
			throw new \XF\PrintableException(\XF::phrase('error_occurred_while_connecting_with_x',
				['provider' => 'Discord']) . ': ' . $e->getMessage());
		}
		
		return $json;
	}
	
	public function patchMember($user, $discord, $remove = false)
	{
		$provider = $this->assertSetup();
		$server = $this->getServer();
		$options = $this->options();
		
		if (!$provider || !$server)
		{
			return false;
		}
		
		$old = $this->getGuild('/members/'.$discord->provider_key);
		$new = [];
		$newGroups = [];
		
		if (!empty($old))
		{
			$old['nick'] = $old['nick'] ?: $old['user']['username'];
			
			if ($options->EWRdiscord_sync_nicks && $old['nick'] != $user->username)
			{
				$new['nick'] = $user->username;
			}
			
			if ($options->EWRdiscord_sync_roles)
			{
				$new['roles'] = [];
				
				foreach ($old['roles'] AS $role)
				{
					if (empty($options->EWRdiscord_roles[$role]['id']))
					{
						$new['roles'][] = $role;
					}
				}
				
				foreach ($options->EWRdiscord_roles AS $role => $group)
				{
					if (!empty($group['id']))
					{
						if ($group['dir'] == 'd2x')
						{
							if (!empty($old['roles']) && in_array($role, $old['roles']))
							{
								$newGroups[] = $group['id'];
								$new['roles'][] = $role;
							}
						}
						else
						{
							if (!$remove && $user->isMemberOf($group['id']))
							{
								$new['roles'][] = $role;
							}
						}
					}
				}
				
				sort($old['roles']);
				sort($new['roles']);
				if ($old['roles'] == $new['roles'])
				{
					unset($new['roles']);
				}
			}
		}
		
		$changeService = \XF::service('XF:User\UserGroupChange');
		$changeGroups = $changeService->getUserGroupChangesForUser($user->user_id);
		$oldGroups = !empty($changeGroups['ewr_discord']) ? explode(',', $changeGroups['ewr_discord']) : [];
		
		sort($oldGroups);
		sort($newGroups);
		if ($oldGroups != $newGroups)
		{
			$this->getLogRepo()->writeLog($user, 'd2x_groups', implode(',', $newGroups), implode(',', $oldGroups));
			
			if (!empty($newGroups))
			{
				$changeService = \XF::service('XF:User\UserGroupChange');
				$changeService->addUserGroupChange($user->user_id, 'ewr_discord', $newGroups);
			}
			else
			{
				$changeService = \XF::service('XF:User\UserGroupChange');
				$changeService->removeUserGroupChange($user->user_id, 'ewr_discord');
			}
		}
		
		
		if (!empty($new))
		{
			if (!empty($new['nick']))
			{
				$this->getLogRepo()->writeLog($user, 'x2d_nick', $new['nick'], $old['nick']);
			}
			
			if (!empty($new['roles']))
			{
				$this->getLogRepo()->writeLog($user, 'x2d_roles', implode(',', $new['roles']), implode(',', $old['roles']));
			}
		
			try
			{
				$client = \XF::app()->http()->client(['timeout' => 5]);
				$response = $client->patch('https://discord.com/api/guilds/'.$server.'/members/'.$discord->provider_key, [
					'headers' => [
						'Authorization' => 'Bot '.$provider->options['bot_token'],
						'User-Agent' => 'DiscordBot (8wayrunBot, v2)',
						'Content-Type' => 'application/json'
					],
					'body' => json_encode($new),
				]);
			}
			catch (\Exception $e) {}
		}
	}
	
	public function patchMemberBan($user, $discord, $ban = true)
	{
		$provider = $this->assertSetup();
		$server = $this->getServer();
		
		if (!$provider || !$server)
		{
			return false;
		}
		
		try
		{
			$client = \XF::app()->http()->client(['timeout' => 5]);
			$payload = [
				'headers' => [
					'Authorization' => 'Bot '.$provider->options['bot_token'],
					'User-Agent' => 'DiscordBot (8wayrunBot, v2)',
				],
				'query' => [
					'reason' => \XF::phrase('EWRdiscord_ban_sync_from_xenforo'),
				],
			];
			
			if ($ban)
			{
				$this->getLogRepo()->writeLog($user, 'x2d_ban', 0, 1);
				$response = $client->put('https://discord.com/api/guilds/'.$server.'/bans/'.$discord->provider_key, $payload);
			}
			else
			{
				$this->getLogRepo()->writeLog($user, 'x2d_ban', 1, 0);
				$response = $client->delete('https://discord.com/api/guilds/'.$server.'/bans/'.$discord->provider_key, $payload);
			}
		}
		catch (\Exception $e) {}
	}
	
	public function createDMChannel($recipient)
	{
		$provider = $this->assertSetup();
		$server = $this->getServer();
		
		if (!$provider || !$server)
		{
			return false;
		}
		
		try
		{
			$client = \XF::app()->http()->client(['timeout' => 5]);
			$response = $client->post('https://discord.com/api/users/@me/channels', [
				'headers' => [
					'Authorization' => 'Bot '.$provider->options['bot_token'],
					'User-Agent' => 'DiscordBot (8wayrunBot, v2)',
					'Content-Type' => 'application/json'
				],
				'body' => json_encode([
					'recipient_id' => $recipient
				]),
			]);
			
			$json = \GuzzleHttp\json_decode($response->getBody(), true);
			return $json['id'];
		}
		catch (\Exception $e) {}
	}
	
	public function postToChannel($channel, $data, $endpoint = 'channels/')
	{
		$provider = $this->assertSetup();
		$server = $this->getServer();
		
		if (!$provider || !$server)
		{
			return false;
		}
		
		try
		{
			$client = \XF::app()->http()->client(['timeout' => 5]);
			$response = $client->post('https://discord.com/api/'.$endpoint.$channel.'/messages', [
				'headers' => [
					'Authorization' => 'Bot '.$provider->options['bot_token'],
					'User-Agent' => 'DiscordBot (8wayrunBot, v2)',
					'Content-Type' => 'application/json'
				],
				'body' => json_encode($data),
			]);
		}
		catch (\Exception $e) {}
	}
	
	public function syncDiscord()
	{
		$discord = $this->finder('XF:UserConnectedAccount')
			->with('User', true)
			->where('provider', 'ewr_discord')
			->where('ewr_discord_syncdate', '<', \XF::$time - (3600 * $this->options()->EWRdiscord_syncbuffer))
			->order('ewr_discord_syncdate', 'ASC')
			->fetchOne();
			
		if ($discord)
		{
			$this->patchMember($discord->User, $discord);
			$discord->set('ewr_discord_syncdate', \XF::$time);
			$discord->save();
		}
		
		try
		{
			$client = \XF::app()->http()->client(['timeout' => 5]);
			$response = $client->get('https://discord.com/api/guilds/'.$this->getServer().'/widget.json');
			$json = \GuzzleHttp\json_decode($response->getBody(), true);
			
			$cache = [
				'server' => $json['id'],
				'invite' => $json['instant_invite'],
				'online' => $json['presence_count'],
			];
			
			\XF::app()->simpleCache()->setValue('EWR/Discord', 'ewr_discord', $cache);
		}
		catch (\Exception $e) {}
	}
	
	public function getProvider()
	{
		return $this->repository('XF:ConnectedAccount')->findProvidersForList()
			->where('provider_id', 'ewr_discord')
			->fetchOne();
	}
	
	public function getServer()
	{
		return $this->options()->EWRdiscord_server;
	}
	
	public function getLogRepo()
	{
		return $this->repository('EWR\Discord:Log');
	}
	
	public function parseChannels($channels)
	{
		usort($channels, function($a, $b) { return($a['position'] > $b['position']); });
			
		$choices = [];
		$parent = [];
		$pcount = 0;
		
		if (!empty($channels) && isset($channels[0]['position']))
		{
			foreach ($channels AS $key => $channel)
			{
				if ($channel['type'] == 4)
				{
					$pcount += 1000;
					$parent[$channel['id']] = $pcount;
					
					$channel['disabled'] = true;
					$choices[$pcount] = $channel;
					unset($channels[$key]);
				}
			}
			
			foreach ($channels AS $channel)
			{
				if ($channel['parent_id'])
				{
					$key = $parent[$channel['parent_id']] + $channel['position'] + 1;
				}
				else
				{
					$key = $channel['position'];
				}
				
				if (!in_array($channel['type'], [0,5]))
				{
					$channel['disabled'] = true;
					$channel['name'] = '< ' . $channel['name'];
				}
				else
				{
					$channel['name'] = '# ' . $channel['name'];
				}
				
				$choices[$key] = $channel;
			}
		}
		
		ksort($choices);
		return $choices;
	}
}